import { z } from "zod";
import { RJSVM_Implementations, RJSVM_Interface, RJSVM, RJSVM_Config } from "../src/RJSVM/framework/types"
import { RJSVM_Builder } from "../src/main";
import { Wallet } from "xrpio";
import { Datawriter } from "./RJSVM/datawriter/datawriter";
import { makeTestnetWallet } from "../test/tools";
import { xrp_transaction_hash_schema } from "./RJSVM/framework/schemas";

const xrpNode = "wss://s.altnet.rippletest.net:51233"

let ownerWallet: Wallet //the RJSVM owner
let userWallet: Wallet //a RJSVM user
let listeningWallet: Wallet //wallet the RJSVM listens to
let drainWallet: Wallet //random wallet to send stuff to, could be anything
let rjsvm: RJSVM

let user_datawriter: Datawriter
let owner_datawriter: Datawriter


const setup = async () => {

    //    [ownerWallet, listeningWallet, userWallet, drainWallet] = await Promise.all([makeTestnetWallet(),makeTestnetWallet(),makeTestnetWallet(),makeTestnetWallet()])
    [ownerWallet, listeningWallet, userWallet, drainWallet] = [
        {
            secret: 'sEd7tPDHFk3w7ABSaCPYi12iwucMQLt',
            address: 'rDML7QhJeVxZTwsjPtm4iyLUJTCzzuppMo'
        },
        {
            secret: 'sEdSgbyouvLM26vq2F2XthfKGWndP72',
            address: 'rPvwbb4JHncG9rKVrYATJw6HRixR69wgjg'
        },
        {
            secret: 'sEd7c1Y8JwWyA2owbchMWnuzAhB3ypY',
            address: 'rwuuWdyxwf3o78Ce2DAZrYCQ12SSawTp6M'
        },
        {
            secret: 'sEdScantwGA5gYe4atKPbG5bsriURpf',
            address: 'rDiwSioTRyV29rNk2FskxqFtMaRsmWvS7C'
        }
    ]

    owner_datawriter = new Datawriter({
        receiveAddress: drainWallet.address,
        sendWallet: ownerWallet,
        xrpNode: xrpNode,
        contractAddress: listeningWallet.address
    })

    user_datawriter = new Datawriter({
        receiveAddress: drainWallet.address,
        sendWallet: userWallet,
        xrpNode: xrpNode,
        contractAddress: listeningWallet.address
    })

    //await owner_dw.callEndpoint('submit', { title: "1", body: "2", from: "3" }, 100)
    //await owner_dw.callEndpoint('submit', { title: "1", body: "2", from: "3" }, 100)
    //await user_dw.callEndpoint('restricted', { title: "user", body: "user", from: "user" })
    //await owner_dw.callEndpoint('restricted', { title: "owner", body: "owner", from: "owner" })

    // #########################
    // Define parameter types
    // #########################

    const shoutSchema = z.object({
        title: z.string(),
        body: z.string(),
        from: z.string(),
        id: z.optional(z.string())
    })
    type Shout = z.infer<typeof shoutSchema>

    const tnsEntrySchema = z.object({
        hash: xrp_transaction_hash_schema,
        name: z.string(),
    })
    type TnsEntry = z.infer<typeof tnsEntrySchema>

    type State = {
        shouts: Shout[]
    }

    // #########################
    // Define endpoints
    // #########################

    type RJSVM_Endpoints = {
        submit: (data: Shout) => void
        restricted: (data: Shout) => void
        setTns: (entry: TnsEntry) => void
    }

    // #########################
    // Define init state
    // #########################

    abstract class RJSVM_Base
        extends RJSVM<State, RJSVM_Endpoints>
        implements RJSVM_Interface<RJSVM_Base> {
        owner = ownerWallet.address

        state: State = {
            shouts: []
        }
    }

    // #########################
    // Implement logic
    // #########################

    const RJSVM_Contract: RJSVM_Implementations<RJSVM_Base> = {
        submit: {
            implementation: function (env, shout) {
                this.state.shouts.unshift(shout)
                console.log(shout)
            },
            visibility: 'public',
            fee: 10,
            parameterSchema: shoutSchema
        },

        restricted: {
            implementation: function (env, shout) {
                this.state.shouts.unshift(shout)
                console.log(shout)
            },
            visibility: 'owner',
            parameterSchema: shoutSchema
        },
        setTns: {
            implementation: function (env, tnsEntry) {
                console.log(tnsEntry)
            },
            visibility: 'public',
            parameterSchema: tnsEntrySchema,
        }
    }

    // #########################
    // Build and connect
    // #########################

    const Rjsvm = RJSVM_Builder.from(RJSVM_Base, RJSVM_Contract);

    const conf: RJSVM_Config = {
        listeningAddress: listeningWallet.address,
        rippleNode: "wss://s.altnet.rippletest.net:51233"
    }

    rjsvm = new Rjsvm(conf)
    await rjsvm.connect()
}

(async () => {
    await setup()
    await owner_datawriter.callEndpoint('submit', { title: "1", body: "2", from: "3" }, 100)
})()